#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <assert.h>

#include "types.h"
#include "points.h"
#include "objects.h"
#include "rand.h"
#include "recipes.h"

#define pi  3.141592653589793238462643

typedef struct 
{
  double x,y,z;
} bubbles;

/* declarations */
void bubble(double***, parlist*);

void read_bubble(bubbles**,int*,parlist*);
void bubble_subsample(bubbles*, int, int);
boolean bubble_read_line(FILE*, char*);
void bubble_flip_bub(bubbles*, bubbles*);

/* ----------------------------------------------- */
/*   Reads the information from the file           */
/* ----------------------------------------------- */
void read_bubble(bubbles**bub, int*n_obj, parlist*par)
{
  FILE  *fp;
  int  i, j;
  char str1[256], str2[256];

  /* open file */
  fprintf(par->outfp, "# using file %s\n", par->infile);
  fp = fopen(par->infile,"r");
  if (fp == NULL) {
    printf("Open file error in read_bubble");
    exit(1);
  }

  /* determine the number of valid lines */
  (*n_obj) = 0;
  while(bubble_read_line(fp, str1) == true) {
    (*n_obj)++;
  }
  fclose(fp);
  /* fprintf(stdout, "%d\n", *n_obj); */

  /* allocate memory */
  *bub = (bubbles*)calloc(*n_obj, sizeof(bubbles));

  fp = fopen(par->infile,"r");
  /* read the coordinates  */
  for (j=0; j<*n_obj; j++) {
    bubble_read_line(fp, str1);
    for (i=0; i<3; i++){
      if (i == 0) strcpy(str2,strtok(str1," \t")); /* read first token */
      else strcpy(str2,strtok(NULL," \t")); /* read next token  */
      
      if ((str2 == NULL) || (strcmp(str2,"\n") == 0)) {
	fprintf(stderr,"Line read-error in read_points : too few columns !");
	fclose(fp);
	return;
      }
      
      switch(i) {
      case 0 : (*bub)[j].x    = strtod(str2, (char**)NULL);
      case 1 : (*bub)[j].y    = strtod(str2, (char**)NULL);
      case 2 : (*bub)[j].z    = strtod(str2, (char**)NULL);
      }
    }
  }  
  /* number of read in bubbles */
  *n_obj = j;

  fclose(fp);

  return; 
}

/* ------------------- */
/*   the catalog       */
/*                     */
/*   may alter par->n  */
/* ------------------- */
void bubble(double ***p, parlist*par)
{
  int    i;
  int    n_bub, n_used;
  bubbles *bub;
  double sim_scale, sim_offset, phys_scale;
  double tmp;
  static boolean first=true;

  /* simulation data is in the range */
  /* bubbles: 56 ; particles: 57 */
  sim_scale  = 128.;
  sim_offset = 0;
  /* the scaling of [0,1] to physical scale */
  phys_scale = sim_scale;

  /* write comments to output file */
  if(first) fprintf(par->outfp, "# Comments \n#\n");
  /* read in the points from par.infile and allocate memory */
  read_bubble(&bub, &n_bub, par);
  par->nin = n_bub;
  if(first) fprintf(par->outfp, "#  %d points in the input file \n", n_bub);
  /* subsample n_used points from n_bub */
  if(par->subsample <= 1.) {
    /* percentage */
    n_used = par->subsample * n_bub;
    tmp = (par->subsample/100. * (double)n_bub) - (double)n_used;
    if(1.1 > tmp && tmp > 0) n_used++;
  } else {
    /* absolute subsampling */
    n_used = par->subsample;
  }
  bubble_subsample(bub, n_used, n_bub);
  if(first) fprintf(par->outfp, "#  subsampled %d from %d points\n", n_used, n_bub);

  /* allocate memory for points */
  par->n = n_used;
  if(first) {
    *p = dmatrix(0, par->n -1, 0,2); 
    assert(*p != NULL);
  }

  /* write to stdout */
  /*
  for(i = 0; i < par->n; i++) {
    fprintf(stdout, "%g  %g  %g\n", 
	    bub[i].x, bub[i].y, bub[i].z);
  }
  exit(0);
  */

  /* rescale and shift within [0,1]^3 */
  /* scaling factor */
  par->scale = phys_scale;
  if(first) fprintf(par->outfp, "#\n#  now we rescale\n");
  if(first) fprintf(par->outfp,"#  scaling factor                     = %g\n", 
		    par->scale);
  for(i = 0; i < par->n; i++) {
    (*p)[i][0] = (bub[i].x + sim_offset)/sim_scale;
    (*p)[i][1] = (bub[i].y + sim_offset)/sim_scale;
    (*p)[i][2] = (bub[i].z + sim_offset)/sim_scale;
  }


  /* free memory etc. */
  free((void*)bub);
  fprintf(stderr, "sampled %d points from %d\n", par->n, n_bub);
  if(first) {
    fprintf(par->outfp, "#\n");
    fflush(par->outfp);
  }
  first = false;

  return;
}

/* -------------------------------------------- */
/*                                              */
/* subsample n_used bubbles from the first     */
/* n_bub bubbles, shuffle the bubbles to the  */
/* n_used first positions in bub                */
/* -------------------------------------------- */
void bubble_subsample(bubbles *bub, int n_used, int n_hem)
{
  int i, irand;

  /* nothing to do */
  if(n_used == n_hem) return;

  /* draw the bubbles */
  for(i=0; i<n_used; i++) {
    irand = i + (int)((n_hem - i)* rand1() - 10e-13);
    bubble_flip_bub(&bub[i],&bub[irand]);
  }

  return;
}
/* flips two bubbles */
void bubble_flip_bub(bubbles *g1, bubbles *g2)
{
  bubbles g;

  g.x = g2->x; g.y = g2->y; g.z = g2->z;
  g2->x = g1->x; g2->y = g1->y; g2->z = g1->z;
  g1->x = g.x; g1->y = g.y; g1->z = g.z;

  return;
}

/*-----------------------------------------------------------------*/
/*   reads in one line from a file, ignores commented lines        */
/*   and lines containing only "\n"                                */
/*                                                                 */
/*   Parameter : fp    : file pointer                              */
/*               str   : pointer to string of minimal length 255   */
/*                                                                 */
/* --------------------------------------------------------------- */
boolean bubble_read_line(FILE *fp, char *line)
{
  char *ptr;

  /* loop till one valid line is read in, exit with FALSE when EOF */
  do {
    if ( fgets(line, 255, fp) == NULL) return false;
    /* search for comment sign #, throw away the rest of the line */
    if ((ptr = strchr(line, '#')) != NULL) *ptr = '\0';
    /* throw away newline \n and anything past */
    if ((ptr = strchr(line, '\n')) != NULL) *ptr = '\0';
  }
  while((strlen(line) == 0));
  return true;
}

