#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>

#include "types.h"
#include "rand.h"
#include "geometry.h"
#include "recipes.h"

#define pi 3.141592653589793238462643
#define epsilon 1e-10

/* Declarations */
void binomial(double***, parlist*);
void patternbinomial(double***, parlist*, pattern*);

/* ----------------------------------------- */
/*                                           */
/*   randomly distribute n points in volume  */
/*   (since n is fixed this is called a      */
/*   binomial distribution, not Poisson)     */
/* ----------------------------------------- */
void binomial(double***p, parlist*par)
{
  int i,j;
  static boolean first=true;

  /* get memory */
  if(first) {
    *p = dmatrix(0,par->n-1,0,2); 
    assert(*p!=NULL);
    first=false;
  }

  for(i=0; i<par->n; i++) for(j=0; j<3; j++) (*p)[i][j] = rand1();
}

/* ---------------------------------------------------- */
/*                                                      */
/* generate random points within window of pattern      */
/* ---------------------------------------------------- */
void patternbinomial(double***p, parlist*par, pattern*pat)
{
  int i,j;
  boolean outside;
  static boolean first=true;

  /* write comments to output file */
  if(first) fprintf(par->outfp, 
		    "# Used binomial distribution within the pattern\n#\n");
  /* get memory */
  if(first) {
    *p = dmatrix(0,par->n-1,0,2); 
    assert(*p!=NULL);
    first=false;
  }

  for(i=0; i<par->n; i++) {
    do {
      /* make random point */
      for(j=0; j<3; j++) (*p)[i][j] = rand1();
      /* test if inside window */
      for(j=0, outside=false; j<pat->nwin && !outside; j++)
	if(bodydist(&pat->win[j],(*p)[i]) > epsilon) outside=true;
    } 
    while(outside);
  }

  return;
}








