#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "types.h"

/* declaration */
double rand1(void);

/* --------------------------------------------------------------- */
/* multiple recursive generator MRGk5-93 from                      */
/*                                                                 */
/*  L'Ecuyer, Blouin, Couture:                                     */
/*  A search for good multiple recursive random number generators  */
/*  ACM Trans. Modeling and Comp. Sim. Vol.3, No. 2, p 87-98 (1993)*/
/*                                                                 */
/* --------------------------------------------------------------- */ 
double rand1( ) {
  unsigned long long a1,a5,m, s1,s5;
  double Invmp1=4.656612873077393e-10; /* 1/(m+1) */

  static unsigned long long x1,x2,x3,x4,x5; 
  static boolean first=true;

  /* initialize */
  if(first) {
    x1=1; x2=2; x3=3; x4=4; x5=5;
    first=false;
  }

  m  = 2147483647;  /* m=2^31-1 */
  a1 = 107374182;
  a5 = 104480;

  /* the congruence part I */
  s1 = (a1*x1) % m;
  s5 = (a5*x5) % m;
  /* shift */
  x5=x4; x4=x3; x3=x2; x2=x1;
  /* the congruence part II */
  x1 = (s1 + s5) % m;

  /* return uniform in (0,1) */
  if(x1 == 0) 
    return m * Invmp1;
  else 
    return x1 * Invmp1;
}
