/* ------------------ */
/* a few basics       */
/* ------------------ */
/* true and false */
typedef enum boolean {false, true} boolean;
/* parameters read from command line */
typedef struct
{
  double lorad, hirad, drad;	/* limits and step for radius */
  int n;		        /* number of points in box */
  int length;                   /* snumber of samples */
  boolean rmbound;              /* flag for deconvolution of boundary */

  char*outfile;			/* output file for Minkowski measures */
  FILE*outfp;                   /* file pointer to the output file */

  char*infile;			/* inputfile for point data */
  int nin;                      /* data points in the input file */

  int hemisphere;		/* 1 for northern, 2 for southern hemisphere */
                                /* of the cluster sample */
  double vmax;                  /* maximal velocity for volume limitation */
  double scale;                 /* the scaling for the radius */
  double subsample;             /* percentage of galaxies for subsampling */

} parlist;

/* --------------------------------------------- */
/*                                               */
/* data structures for the objects               */
/* involved in calculating Minkowski functionals */
/* --------------------------------------------- */
/* a twodimensional body */
typedef enum bodytype {nobody, ball, hole, wall} bodytype;
typedef struct body
{
  bodytype type;		/* type of body (see above) */
  int n;			/* number */
  struct body*touch;		/* pointer to neighbouring body */
  double**par;			/* parameters according to type */
  boolean wurscht;		/* body is completely irrelevant */
  double*v;			/* partial Minkowski functionals */
} body;

/* a configuration of some bodies in some window */
typedef struct pattern
{
  body*b;			/* list of individual bodies */
  body*win;                     /* list of the original windows */
  int n;			/* total number of bodies */
  int nwin;			/* number of bodies for window */
} pattern;

/* the intersection curve of two bodies */
typedef enum secttype {nosect, circ, line} secttype;
typedef struct section
{
  secttype type;		/* type of curve (see above) */
  double**par;			/* parameter list according to type */
  body**parent;			/* pointers to intersecting bodies */
  double*v;			/* local Minkowski functionals */
} section;

/* the intersection point of three bodies */
typedef struct triple
{
  double*x;			/* coordinates */
  double par;			/* parameter on section line */
  body**parent;			/* the three bodies */
  struct triple*next,*prev;	/* pointers for ring list */
} triple;

